#!/usr/bin/python

import os
import subprocess
import sys

from charmhelpers.core import hookenv
from charmhelpers import fetch
from charmhelpers.fetch import archiveurl
from path import path


hooks = hookenv.Hooks()
RELEASE_URL = 'https://github.com/hashicorp/consul-template/releases/download'
SHA256SUMS = {
    'consul-template_0.9.0_linux_amd64.tar.gz':
    '3d8c9fcaee18a4369cc731528ce9d6a5be03d88b954a2fea0f4406fc54c70fc8'
}


@hooks.hook('template-relation-joined')
def consul_joined():
    """
    Called when the charm initiates a consul relation.
    Install consul agent and consul-template,
    Start the consul services with the consul server information.
    Service discovery is done by consul server.
    """
    hookenv.log('Entering template-relation-joined.')
    install_consul_template('0.9.0')

    # Configure consul-template to run for haproxy.
    address = hookenv.relation_get('address')
    port = hookenv.relation_get('port')
    consul_server = '{0}:{1}'.format(address, port)
    print(consul_server)

    # Write consul-template configuration.
    write_consul_template_config(consul_server)
    host.service_restart('consul-template')

    hookenv.log('Exited template-relation-joined.')


@hooks.hook('template-relation-changed')
def consul_changed():
    """
    Called when a value changes on the consul relationship.
    consul-template updates automatically when values change on consul server,
    so this method may not be needed.
    """
    hookenv.log('Entering template-relation-changed.')
    address = hookenv.relation_get('address')
    port = hookenv.relation_get('port')
    consul_server = '{0}:{1}'.format(address, port)
    print(consul_server)

    # Write consul-template configuration.
    write_consul_config(consul_server)
    host.service_restart('consul-template')

    hookenv.log('Exited template-relation-changed.')


@hooks.hook('template-relation-departed')
def consul_departed():
    """
    Called when Juju is about to break a relation.
    Consul-template may need to exit or leave the consul relation.
    """
    hookenv.log('Entering template-relation-departed.')

    host.service_stop('consul-template')
    # We should reinstate the "standard" haproxy configuration here.

    hookenv.log('Exited template-relation-departed.')


@hooks.hook('template-relation-broken')
def consul_broken():
    """
    Called when there is no longer a relationship between services.
    Is there anything to monitor?
    Do we need to keep consul-template running at this point?
    """
    hookenv.log('Entering template-relation-broken.')
    hookenv.log('Exited template-relation-broken.')


def write_consul_config(consul_server):
    """
    Write the configuration file for the consul-template service.
    """
    source = hookenv.charm_dir() + '/files/consul/haproxy.ctmpl'
    destination = '/etc/haproxy/haproxy.cfg'
    variables = {'consul_sever': consul_server, 'source': source,
                 'destination':destination}
    configuration = hookenv.charm_dir() + '/files/consul/consul-template.json'
    with open(configuration) as stream:
        content = stream.read()
    # Replace variables
    rendered = content % variables
    with open('/etc/consul-template.conf', 'w') as stream:
        stream.write(rendered)


def consul_arch():
    ''' Generate the consul string based on platform values of this system. '''
    import platform
    # Find the system type string and convert to lower case.
    system = platform.system().lower()
    # Consul builds are only available for 'darwin' and 'linux'
    if system in ['darwin', 'linux']:
        machine = platform.machine()
        # Consul builds are only available for 'amd64' and '386'
        if machine == 'x86_64':
            package_arch = 'amd64'
        elif machine == 'i386':
            package_arch = '386'
        else:
            raise Exception('Invalid machine type: {0}'.format(machine))
        if package_arch:
            architecture = '{0}_{1}'.format(system, package_arch)
            return architecture
    else:
        raise Exception('Invalid system type: {0}'.format(system))


def install_consul_template(version, destination_directory='/usr/local/bin'):
    """
    Install the desired version of consul-template for this architecture.
    """
    if version:
        architecture = consul_arch()
        name = 'consul-template_{0}_{1}.tar.gz'.format(version, architecture)
        # Find the sha256sum for the specific file name.
        sha256sum = SHA256SUMS[name]
        print('Expecting {0} for {1}'.format(sha256sum, name))
        # Build the full url with release/version/name
        url = '{0}/v{1}/{2}'.format(RELEASE_URL, version, name)
        print('Fetching {0}'.format(url))
        installer = archiveurl.ArchiveUrlFetchHandler()
        installer.install(url, dest=destination_directory, checksum=sha256sum,
                          hash_type='sha256')
        unpacked_directory = path(destination_directory +
            '/consul-template_{0}_{1}'.format(version, architecture))
        consul_template = path(unpacked_directory + '/consul-template')
        print('Copying {0} to {1}'.format(consul_template, destination_directory))
        consul_template.copy(destination_directory)
        # Make the consul-template binary executable.
        consul_template.chmod(0o555)
        unpacked_directory.removedirs_p()


if __name__ == '__main__':
    hooks.execute(sys.argv)
